@@ -19,19 +19,18 @@ impl Loader {
     }
 
     pub fn load(&self) -> Result<Config> {
-        if let Some(path) = self.xdg_dirs.find_config_file(format!("{}.toml", NAME)) {
-            self.load_path(&path)
-        } else {
-            Ok(Config::default())
-        }
+        self.xdg_dirs
+            .find_config_file(format!("{}.toml", NAME))
+            .as_deref()
+            .map_or_else(|| Ok(Config::default()), Self::load_path)
     }
 
-    pub fn load_path(&self, path: &Path) -> Result<Config> {
+    pub fn load_path(path: &Path) -> Result<Config> {
         let data = std::fs::read_to_string(&path).context("Config file")?;
         Ok(toml::from_str(&data).context("Config Toml")?)
     }
 
-    pub fn print(&self, cfg: &Config) -> Result<()> {
+    pub fn print(cfg: &Config) -> Result<()> {
         let toml = toml::to_string_pretty(cfg).context("toml serialize")?;
         std::io::stdout()
             .write_all(toml.as_bytes())
@@ -98,7 +97,7 @@ impl std::str::FromStr for Rgba {
         let without_prefix = s.trim_start_matches('#');
         match without_prefix.len() {
             8 => {
-                let mut bytes = [0u8; 4];
+                let mut bytes = [0_u8; 4];
                 hex::decode_to_slice(without_prefix, &mut bytes).context("color")?;
                 log::trace!("rgba::from_str {:?}", bytes);
                 Ok(Rgba {
@@ -109,7 +108,7 @@ impl std::str::FromStr for Rgba {
                 })
             }
             6 => {
-                let mut bytes = [0u8; 3];
+                let mut bytes = [0_u8; 3];
                 hex::decode_to_slice(without_prefix, &mut bytes).context("color")?;
                 log::trace!("rgba::from_str {:?}", bytes);
                 Ok(Rgba {
@@ -157,7 +156,11 @@ pub struct Dialog {
     #[serde(serialize_with = "option_explicit_serialize")]
     #[serde(deserialize_with = "option_explicit_none")]
     pub font_file: Option<std::ffi::CString>,
+    #[serde(serialize_with = "option_explicit_serialize")]
+    #[serde(deserialize_with = "option_explicit_none")]
+    pub direction: Option<PangoDirection>,
     pub label: String,
+    pub alignment: PangoAlignment,
     #[serde(serialize_with = "option_explicit_serialize")]
     #[serde(deserialize_with = "option_explicit_none")]
     pub scale: Option<f64>,
@@ -199,9 +202,11 @@ impl Default for Dialog {
             indicator_label_foreground: "#5c616c".parse().unwrap(),
             background: "#f5f6f7ee".parse().unwrap(),
             label: "Please enter your authentication passphrase:".into(),
+            alignment: PangoAlignment::Left,
             indicator_label: "Secret:".into(),
             input_timeout: Some(30),
-            font: Some("11".into()),
+            font: Some("mono 11".into()),
+            direction: None,
             scale: None,
             font_file: None,
             layout_opts: Layout::default(),
@@ -320,19 +325,13 @@ pub struct Layout {
 }
 
 impl Layout {
-    pub fn horizontal_spacing(&self, text_height: u32) -> f64 {
-        if let Some(horizontal_spacing) = self.horizontal_spacing {
-            horizontal_spacing
-        } else {
-            (text_height as f64 / 1.7).round()
-        }
+    pub fn horizontal_spacing(&self, text_height: f64) -> f64 {
+        self.horizontal_spacing
+            .unwrap_or_else(|| (text_height / 1.7).round())
     }
-    pub fn vertical_spacing(&self, text_height: u32) -> f64 {
-        if let Some(vertical_spacing) = self.vertical_spacing {
-            vertical_spacing
-        } else {
-            (text_height as f64 / 1.7).round()
-        }
+    pub fn vertical_spacing(&self, text_height: f64) -> f64 {
+        self.vertical_spacing
+            .unwrap_or_else(|| (text_height / 1.7).round())
     }
 }
 
@@ -347,7 +346,7 @@ impl Default for Layout {
     }
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize, Copy)]
 #[serde(default)]
 pub struct IndicatorClassic {
     pub min_count: u16,
@@ -379,7 +378,7 @@ impl Default for IndicatorClassic {
     }
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize, Copy)]
 #[serde(default)]
 pub struct IndicatorCircle {
     #[serde(serialize_with = "option_explicit_serialize")]
@@ -404,7 +403,7 @@ impl Default for IndicatorCircle {
             rotate: true,
             light_up: true,
             rotation_speed_start: 0.10,
-            rotation_speed_gain: 1.2,
+            rotation_speed_gain: 1.05,
             spacing_angle: 0.5,
             indicator_count: 3,
             indicator_width: None,
@@ -448,6 +447,27 @@ impl From<PangoAlignment> for pango::Alignment {
     }
 }
 
+#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
+pub enum PangoDirection {
+    Ltr,
+    Neutral,
+    Rtl,
+    WeakLtr,
+    WeakRtl,
+}
+
+impl From<PangoDirection> for pango::Direction {
+    fn from(val: PangoDirection) -> Self {
+        match val {
+            PangoDirection::Ltr => Self::Ltr,
+            PangoDirection::Neutral => Self::Neutral,
+            PangoDirection::Rtl => Self::Rtl,
+            PangoDirection::WeakLtr => Self::WeakLtr,
+            PangoDirection::WeakRtl => Self::WeakRtl,
+        }
+    }
+}
+
 fn strings<'de, D>(d: D) -> std::result::Result<Vec<String>, D::Error>
 where
     D: serde::de::Deserializer<'de>,
@@ -473,6 +493,7 @@ pub struct Custom {
     pub strings: Vec<String>,
 }
 
+#[allow(clippy::unicode_not_nfc)]
 impl Default for Custom {
     fn default() -> Self {
         Self {
@@ -574,7 +595,7 @@ impl Default for Indicator {
     }
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize, Copy)]
 #[serde(default)]
 pub struct IndicatorCommon {
     pub border_width: f64,
